]> git.sesse.net Git - cubemap/blobdiff - server.h
Rename header_or_error to header_or_short_response, as it will soon be able to contai...
[cubemap] / server.h
index edcdc83438b9a0322f382514601abf71e46f43ff..cfc9d25160413a5ede6408213971a9d6bebed98f 100644 (file)
--- a/server.h
+++ b/server.h
@@ -8,6 +8,7 @@
 #include <sys/types.h>
 #include <time.h>
 #include <map>
+#include <queue>
 #include <string>
 #include <vector>
 
@@ -21,9 +22,9 @@ struct Stream;
 #define EPOLL_MAX_EVENTS 8192
 #define EPOLL_TIMEOUT_MS 20
 #define MAX_CLIENT_REQUEST 16384
+#define REQUEST_READ_TIMEOUT_SEC 60
 
 class CubemapStateProto;
-class MarkPool;
 class StreamProto;
 
 class Server : public Thread {
@@ -35,35 +36,37 @@ public:
        std::vector<ClientStats> get_client_stats() const;
 
        // Set header (both HTTP header and any stream headers) for the given stream.
-       void set_header(const std::string &stream_id,
+       void set_header(int stream_index,
                        const std::string &http_header,
                        const std::string &stream_header);
 
// Set that the given stream should use the given mark pool from now on.
        // NOTE: This should be set before any clients are connected!
-       void set_mark_pool(const std::string &stream_id, MarkPool *mark_pool);
+       void set_pacing_rate(int stream_index, uint32_t pacing_rate);
 
        // These will be deferred until the next time an iteration in do_work() happens,
        // and the order between them are undefined.
        // XXX: header should ideally be ordered with respect to data.
        void add_client_deferred(int sock);
-       void add_data_deferred(const std::string &stream_id, const char *data, size_t bytes);
+       void add_data_deferred(int stream_index, const char *data, size_t bytes, StreamStartSuitability suitable_for_stream_start);
 
        // These should not be called while running, since that would violate
        // threading assumptions (ie., that epoll is only called from one thread
        // at the same time).
        CubemapStateProto serialize();
        void add_client_from_serialized(const ClientProto &client);
-       void add_stream(const std::string &stream_id, size_t bytes_received, Stream::Encoding encoding);
-       void add_stream_from_serialized(const StreamProto &stream, int data_fd);
-       void set_backlog_size(const std::string &stream_id, size_t new_size);
-       void set_encoding(const std::string &stream_id, Stream::Encoding encoding);
+       int add_stream(const std::string &url, size_t bytes_received, size_t prebuffering_bytes, Stream::Encoding encoding);
+       int add_stream_from_serialized(const StreamProto &stream, int data_fd);
+       int lookup_stream_by_url(const std::string &url) const;
+       void set_backlog_size(int stream_index, size_t new_size);
+       void set_prebuffering_bytes(int stream_index, size_t new_amount);
+       void set_encoding(int stream_index, Stream::Encoding encoding);
 
 private:
-       // Mutex protecting queued_add_clients and streams[..]->queued_data.
+       // Mutex protecting queued_add_clients.
        // Note that if you want to hold both this and <mutex> below,
        // you will need to take <mutex> before this one.
-       mutable pthread_mutex_t queued_data_mutex;
+       mutable pthread_mutex_t queued_clients_mutex;
 
        // Deferred commands that should be run from the do_work() thread as soon as possible.
        // We defer these for two reasons:
@@ -74,18 +77,34 @@ private:
        //    add_data(), since they want to do add_data() rather often, and <mutex>
        //    can be taken a lot of the time.
        //      
-       // Protected by <queued_data_mutex>.
+       // Protected by <queued_clients_mutex>.
        std::vector<int> queued_add_clients;
 
        // All variables below this line are protected by the mutex.
        mutable pthread_mutex_t mutex;
 
-       // Map from stream ID to stream.
-       std::map<std::string, Stream *> streams;
+       // All streams.
+       std::vector<Stream *> streams;
+
+       // Map from URL to index into <streams>.
+       std::map<std::string, int> url_map;
 
        // Map from file descriptor to client.
        std::map<int, Client> clients;
 
+       // A list of all clients, ordered by the time they connected (first element),
+       // and their file descriptor (second element). It is ordered by connection time
+       // (and thus also by read timeout time) so that we can read clients from the
+       // start and stop processing once we get to one that isn't ready to be
+       // timed out yet (which means we only have to look at each client exactly once,
+       // save for the first element of the queue, which is always checked).
+       //
+       // Note that when we delete a client, we don't update this queue.
+       // This means that when reading it, we need to check if the client it
+       // describes is still exists (ie., that the fd still exists, and that
+       // the timespec matches).
+       std::queue<std::pair<timespec, int> > clients_ordered_by_connect_time;
+
        // Used for epoll implementation (obviously).
        int epoll_fd;
        epoll_event events[EPOLL_MAX_EVENTS];
@@ -119,13 +138,11 @@ private:
        void construct_header(Client *client);
 
        // Construct a generic error with the given line, and set the client into
-       // the SENDING_ERROR state.
+       // the SENDING_SHORT_RESPONSE state.
        void construct_error(Client *client, int error_code);
 
-       // TODO: This function should probably die.
-       Stream *find_stream(const std::string &stream_id);
-
        void process_queued_data();
+       void skip_lost_data(Client *client);
 
        void add_client(int sock);
 };
@@ -35,35 +36,37 @@ public:
                        const std::string &stream_header);
 
-       // Set that the given stream should use the given mark pool from now on.
+       // Set that the given stream should use the given max pacing rate from now on.
        // NOTE: This should be set before any clients are connected!